from manga_py.provider import Provider
from .helpers.std import Std


class DesuMe(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name({'vol': idx[0], 'ch': idx[1]})

    def get_chapter_index(self) -> str:
        result = self.re.search(r'/vol(\d+)/ch(\d+)', self.chapter).groups()
        return '{}-{}'.format(result[0], result[1])

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_chapters(self):
        return self._elements('#animeView ul h4 > a.tips')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_files(self):
        content = self.http_get(self.domain + self.chapter)
        result = self.re.search(r'images:\s?(\[\[.+\]\])', content, self.re.M)
        if not result:
            return []
        root_url = self.re.search(r'dir:\s?"([^"]*)"', content).group(1).replace(r'\/', '/')

        return [root_url + i[0] for i in self.json.loads(result.group(1))]

    def get_cover(self):
        return self._cover_from_content('.c-poster > img')


main = DesuMe



from abc import abstractmethod, ABCMeta


class ArchiveName(metaclass=ABCMeta):
    _vol_fill = False

    @abstractmethod
    def get_chapter_index(self):
        pass

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index()
        self._vol_fill = True
        return self.normal_arc_name({'vol': idx.split('-')})

    def normal_arc_name(self, idx) -> str:
        if isinstance(idx, (str, int)):
            idx = [idx]
        if isinstance(idx, list):
            self._vol_fill = True
            return self.__normal_name_list(idx)
        if isinstance(idx, dict):
            return self.__normal_name_dict(idx)
        raise DeprecationWarning('Wrong arc name type: %s' % type(idx))

    def __normal_name_dict(self, idx: dict) -> str:
        vol = idx.get('vol', None)
        ch = idx.get('ch', None)
        result = ''
        if vol:
            if isinstance(vol, str):
                vol = [vol]
            result = self.__normal_name_list(vol)
        if ch:
            result += '-ch_' + self.__fill(ch)

        if self._with_manga_name:
            name = self._params.get('name', '')
            if not len(name):
                name = self.manga_name

            result = '%s-%s' % (name, result)

        return result

    def __normal_name_list(self, idx: list) -> str:
        fmt = 'vol_{:0>3}'
        if len(idx) > 1:
            fmt += '-{}' * (len(idx) - 1)
        elif self._vol_fill and self._zero_fill:
            idx.append('0')
            fmt += '-{}'
        return fmt.format(*idx)

    @staticmethod
    def __fill(var, fmt: str = '-{}') -> str:
        if isinstance(var, str):
            var = [var]
        return (fmt * len(var)).format(*var).lstrip('-')



from manga_py.provider import Provider
from .helpers.std import Std
from time import sleep


class MangaTrCom(Provider, Std):

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        idx = self.re.search('-chapter-(.+)\.html', chapter).group(1)
        return '-'.join(idx.split('.'))

    def get_content(self):
        return self._get_content('{}/manga-{}.html')

    def get_manga_name(self) -> str:
        url = self.get_url()
        re = r'\d-read-(.+)-chapter-'
        if ~url.find('/manga-'):
            re = r'/manga-(.+)\.html'
        return self.re.search(re, url).group(1)

    def get_chapters(self):
        return self._elements('#results td.left a')

    def get_files(self):
        img_selector = 'img.chapter-img'
        parser = self.html_fromstring(self.chapter)
        pages = self._first_select_options(parser, '.chapter-content select')
        images = self._images_helper(parser, img_selector)
        n = self.http().normalize_uri
        for i in pages:
            parser = self.html_fromstring(n(i))
            images += self._images_helper(parser, img_selector)
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('img.thumbnail')

    def prepare_cookies(self):
        self.http().requests(self.get_url(), method='head')
        self.log('Anti-ddos...')
        sleep(3)


main = MangaTrCom



from manga_py.provider import Provider
from .helpers.std import Std


class ShakaiRu(Provider, Std):
    _api_url = 'http://shakai.ru/take/api-manga/request/shakai'

    def get_chapter_index(self) -> str:
        idx = self.chapter.get('data-first')
        return idx.replace('_', '-')

    def get_content(self):
        idx = self._get_name(r'/manga[^/]*/(\d+)')
        _ = {
            'dataRun': 'api-manga',
            'dataRequest': idx
        }
        with self.http().post(self._api_url, data=_) as resp:
            return resp.json()

    def get_manga_name(self) -> str:
        parser = self.content.get('post', [])
        idx = self._get_name(r'/manga[^/]*/(\d+)')
        parser = parser[3] if len(parser) > 3 else idx
        return parser.split('/')[0].strip()

    def get_chapters(self):
        return self.content.get('data', [])[::-1]

    def get_files(self):
        chapter = self.chapter
        if isinstance(chapter, dict):
            return chapter.get('data-second', [])
        return []

    def get_cover(self):
        pass  # FIXME HOME

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ShakaiRu



from argparse import Namespace
from datetime import datetime
from sys import argv
from typing import Union

from manga_py import meta


class Info:
    __doc__ = """
    --print-json argument helper
    
    {
        'site': 'https://example.org/kumo-desu-ga-nani-ka',
        'downloader': [
            'https://manga-py.com/manga-py/',
            'https://github.com/manga-py/manga-py',
            'https://github.com/yuru-yuri/manga-py',
            'https://yuru-yuri.github.io/manga-py',
        ],
        'version': '1.1.4',
        'delta': '0:00:00.003625',
        'start': '2018-06-08 17:22:24.419565',
        'end': '2018-06-08 17:22:24.423190',
        'user_agent': 'Mozilla/5.0',
        'cookies': {'cf_clearance': 'ec-1528654923-86400', '__cfduid': '21528654914'},
        'args': {
            '_raw_params': 'manga-py --cbz https://example.org/kumo-desu-ga-nani-ka',
            'url': 'https://example.org/kumo-desu-ga-nani-ka',
            'name': None,
            'destination': None,
            'no-progress': False,
            'cbz': False,
            'skip-volumes': None,
            'max-volumes': None,
            'user-agent': None,
            'proxy': None,
            'reverse-downloading': None,
            'rewrite-exists-archives': None,
            'no-multi-threads': None,
        },
        'error': False,
        'error_msg': '',
        'volumes': [
            {
              'name': 'Kumo desu ga, nani ka? - 0',
              'path': 'Manga/kumo-desu-ga-nani-ka/vol_000.zip',
            },
            {
              'name': 'Kumo desu ga, nani ka? - 1',
              'path': 'Manga/kumo-desu-ga-nani-ka/vol_001.zip',
            },
        ],
    }
    """
    _data = None
    _start_time = None

    @staticmethod
    def _dt(dt, fmt: str = '%A, %d. %B %Y %H:%M:%S'):
        return dt.strftime(fmt)

    def __init__(self, args: Union[Namespace, dict]):  # see manga_py.cli arguments
        _args = args.__dict__ if not isinstance(args, dict) else args
        _args['_raw_params'] = ' '.join(argv)
        self._data = {
            'site': args.url,
            'downloader': meta.repo_url,
            'version': meta.version,
            'delta': None,
            'init': self._dt(datetime.now()),
            'start': None,
            'end': None,
            'user_agent': None,
            'cookies': None,
            'args': _args,
            'return_code': 0,
            'error': False,
            'error_msg': None,
            'volumes': [],
        }
        self._volumes = []

    def set_ua(self, ua):
        self._data['user_agent'] = ua

    def set_error(self, e: bool, message: str, rc: int = 1):
        self._data['return_code'] = rc
        self._data['error'] = e

    def start(self):
        self._start_time = datetime.now()

    def set_cookies(self, cookies):
        self._data['cookies'] = cookies

    def set_volumes(self, volumes: list):
        self._data['volumes'] = volumes

    def set_last_volume_error(self, error_message):
        self.set_error(True, error_message)
        try:
            self._data['volumes'][-1]['error'] = True
            self._data['volumes'][-1]['error_message'] = error_message
        except IndexError:
            pass

    def add_volume(self, url: str, path: str, files: list = None):
        volume = {
            'url': url,
            'path': path,
            'error': False,
            'error_message': '',
        }

        if files is not None:
            volume['files'] = files
            volume['num_files'] = len(files)

        self._data['volumes'].append(volume)

    def get(self):
        self._data['delta'] = str(datetime.now() - self._start_time)
        self._data['start'] = self._dt(self._start_time)
        self._data['end'] = self._dt(datetime.now())
        return self._data



from manga_py.provider import Provider
from .helpers.std import Std
from logging import warning, error
import re
import itertools


RE_APP = re.compile(r'App\s?=\s?({.+?})[;<]')


class ReMangaOrg(Provider, Std):
    def get_archive_name(self) -> str:
        return self.normal_arc_name({
            'vol': str(self.chapter['tome']),
            'ch': str(self.chapter['chapter']).split('.'),
        })

    def get_chapter_index(self) -> str:
        return str(self.chapter['index'])

    def get_content(self):
        js_data = RE_APP.search(self._get_content('{}/manga/{}')).group(1)
        return self.json.loads(js_data)

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        with self.http().get(
            self._api_url('/titles/chapters/?branch_id=%d' % self._branch_id)
        ) as resp:
            data = resp.json()

        paid_chapters = len([ch for ch in data['content'] if ch['is_paid']])
        if paid_chapters:
            self.log('Found %d paid chapters. Please, check site on your browser' % paid_chapters)

        return [ch for ch in data['content'] if not ch['is_paid']]

    def _api_url(self, sub_url: str):
        return '{}{}'.format(
            self.content['apiUrl'].rstrip('/'),
            sub_url
        )

    def get_files(self):
        try:
            with self.http().get(
                self._api_url('/titles/chapters/{}'.format(
                    self.chapter['id']
                ))
            ) as resp:
                pages = resp.json().get('content', {}).get('pages', [])
                if len(pages) == 0:
                    warning('Pages list has empty')
                    return []

                if type(pages[0]) == list:
                    images = list(itertools.chain(
                        *pages
                    ))
                else:
                    images = pages

            return [img['link'] for img in images]

        except Exception as e:
            warning(str(e))
            return []

        # content = self._get_content(
        #     '{}/manga/{}/ch%d' % self.chapter['id']
        # )
        #
        # js_data = RE_APP.search(content).group(1)
        # json_data = self.json.loads(js_data)
        #
        # images = self._content_value([
        #     'state',
        #     'chapter',
        #     'content',
        #     'data',
        #     'pages',
        # ], json_data)
        #
        # return [img['link'] for img in sorted(images, key=lambda x: x['page'])]

    @property
    def _branch_id(self):
        branches = self._content_value([
            'state',
            'manga',
            'content',
            'data',
            'branches',
        ])
        if len(branches) == 0:
            error('Branches list has empty')
            return []
        if len(branches) > 1:
            warning('Branches list too long. Please, report this url to')

        return branches[0]['id']

    def _content_value(self, keys: list, content=None):
        data = content or self.content
        for key in keys:
            data = data.get(key, {})
        return data

    def get_cover(self) -> str:
        return self._content_value([
            'state',
            'manga',
            'content',
            'data',
            'img',
            'high',
        ])


main = ReMangaOrg



from manga_py.provider import Provider
from .helpers.std import Std
from requests import get


class MangaNelosCom(Provider, Std):
    def get_chapter_index(self) -> str:
        ch = self.re.search(r'-chapter-(\d+(?:\.\d+)?)', self.chapter)
        return ch.group(1).replace('.', '-')

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.content .chapter a')

    def get_files(self):
        content = self.text_content(self.http_get(self.chapter), '#arraydata')
        images = content.split(',')

        if len(images) == 0:
            return []

        if self._download_cookies is None:
            with get(images[0], stream=True) as req:
                self._download_cookies = req.cookies

        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.media-left.cover-detail')

    def prepare_cookies(self):
        self.http()._download = self._download


main = MangaNelosCom



author = 'Sergey Zharkov'
license_type = 'MIT'
email = 'sttv-pc@mail.ru'
version = '1.29.1'
repo_name = 'manga-py/manga-py'
repo_url = 'https://github.com/%s' % repo_name

__all__ = [
    'author',
    'license_type',
    'email',
    'version',
    'repo_name',
    'repo_url',
]



from .base_lib import BaseLib
import re
import pathlib


RE_BATOJS = re.compile(r'\w batojs\s*=\s*(.+);')
RE_SERVER = re.compile(r'\w server\s*=\s*"(.+)";')


class BatoToCrypt(BaseLib):
    @classmethod
    def decrypt_server(cls, page_content: str) -> str:  # pragma: no cover
        batojs = RE_BATOJS.search(page_content).group(1)
        server = RE_SERVER.search(page_content).group(1)

        return cls._server(server, batojs)

    @classmethod
    def _server(cls, server, key):
        with pathlib.Path(__file__).parent.absolute().joinpath('aes.js').open('r') as r:
            script = r.read()
        ev = f"""CryptoJS.AES.decrypt("{server}", ({key}).toString()).toString(CryptoJS.enc.Utf8)"""
        return cls.exec_js(script, ev)



from .gomanga_co import GoMangaCo
from .helpers.std import Std


class KomikIdCom(GoMangaCo, Std):
    _name_re = '/manga/([^/]+)'
    _content_str = '{}/manga/{}'
    _chapters_selector = '.chapter-title-rtl a'

    def get_chapter_index(self) -> str:
        re = r'/manga/[^/]+/(\d+)(?:[^\d](\d+))?'
        idx = self.re.search(re, self.chapter)
        return self._join_groups(idx.groups())

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#all img[data-src]', 'data-src')

    def get_cover(self) -> str:
        return self._cover_from_content('.boxed img')

    def prepare_cookies(self):
        pass


main = KomikIdCom



from manga_py.provider import Provider
from .helpers.std import Std
from ..crypt.hentainexus_com import HentaiNexusComCrypt


class HentaiNexusCom(Provider, Std):
    def _code(self):
        return self.re.search(r'/(?:read|view)/(\d+)', self.get_url()).group(1)

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_content(self):
        return self.http_get('{}/view/{}'.format(self.domain, self._code()))

    def get_manga_name(self) -> str:
        return self.text_content(self.content, 'h1.title')

    def get_chapters(self):
        return [b'0']

    def get_files(self):
        content = self.http_get('{}/read/{}'.format(self.domain, self._code()))
        code = self.re.search(r"""initReader\(['"](.+?)['"]""", content).group(1)

        json_data = self.json.loads(self.helper.decode(code))

        server = json_data['b']
        server_path = json_data['r']
        server_code = json_data['i']

        return ['{}{}{}/{}/{}'.format(
            server,
            server_path,
            i['h'],
            server_code,
            i['p']
        ) for i in json_data['f']]

    def prepare_cookies(self):
        self.helper = HentaiNexusComCrypt()

    def get_cover(self) -> str:
        return self._cover_from_content('.is-one-quarter-fullhd  figure.image > img')


main = HentaiNexusCom



from .rawdevart_com_old import RawDevArtComOld


class MangaSushiNet(RawDevArtComOld):
    _chapter_selector = r'/chapter-(\d+(?:-\d+)?)'

    def get_chapter_index(self) -> str:
        idx = self.re.search(self._chapter_selector, self.chapter)
        return idx.group(1)

    def get_files(self):
        chapter = self.chapter.replace('p/1/', '').replace('?style=paged', '') + '?style=list'
        parser = self.html_fromstring(chapter)
        _class = '.page-break img.wp-manga-chapter-img'
        return self._images_helper(parser, _class, 'data-src')


main = MangaSushiNet



from manga_py.provider import Provider
from .helpers.std import Std


class MangaZukiMe(Provider, Std):
    _prefix = '/manga/'

    def get_chapter_index(self) -> str:
        try:
            re = self.re.compile(r'%s[^/]+/.+?(\d+(?:-\d+)?)[\?/]' % self._prefix)
            return re.search(self.chapter).group(1)
        except AttributeError:
            # mangazuki.online
            re = self.re.compile(r'%s[^/]+/.+?(\d+(?:-\d+)?)$' % self._prefix)
            return re.search(self.chapter).group(1)

    def get_content(self):
        return self._get_content('{}%s{}' % self._prefix)

    def get_manga_name(self) -> str:
        return self._get_name('%s([^/]+)' % self._prefix)

    def get_chapters(self):
        chapters = []
        n = self.http().normalize_uri
        re = self.re.compile(r'(.+?)(?:\?style=list)?(?:/)?$')
        for ch in self._elements('.wp-manga-chapter > a'):
            href = re.search(ch.get('href')).group(1)
            chapters.append(n(href) + '?style=list')
        return chapters

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, 'img.wp-manga-chapter-img')

    def get_cover(self) -> str:
        image = self._cover_from_content('.summary_image > a > img', 'data-src')
        if len(image) < 1:
            # mangazuki.online
            image = self._cover_from_content('.summary_image > a > img')
        return image

    def prepare_cookies(self):
        self._prefix = self.re.search('(/mangas?/)', self.get_url()).group(1)

        self.cf_scrape(self.get_url())


main = MangaZukiMe



from manga_py.provider import Provider
from .helpers.std import Std


class AComicsRu(Provider, Std):

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_content(self):
        return self._get_content('{}/~{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/~([^/]+)')

    def get_chapters(self):
        return ['~' + self.manga_name]

    def get_files(self):
        pages_max = self.text_content(self.content, 'span.issueNumber').split('/')[1]
        _min = self._params['skip_volumes']
        _max = self._params['max_volumes']
        if _max > 0 and _min > 0:
            _max += _min - 1

        if _max == 0:
            _max = int(pages_max)

        images = []
        for i in range(_min, _max):
            parser = self.document_fromstring(self._get_content('{}/~{}/%d' % (i + 1)))
            images += self._images_helper(parser, '#mainImage')

        return images

    def get_cover(self) -> str:
        return self._cover_from_content('header.serial a img')

    def book_meta(self) -> dict:
        pass

    def prepare_cookies(self):
        self.update_cookies({'ageRestrict': '21'})


main = AComicsRu



from manga_py.provider import Provider
from .helpers.std import Std


class MangaTailCom(Provider, Std):
    __local_storage = None

    def get_archive_name(self) -> str:
        return self.normal_arc_name([
            self.chapter_id,
            *self._parse_ch(self.chapter[0]).split('.')
        ])

    def get_chapter_index(self) -> str:  # Oh ...
        pass

    def _parse_ch(self, chapter):
        _re = r'.+?[^\d](\d+(?:\.\d+)?)'
        if ~chapter.find('-fixed'):
            _re = r'.+?[^\d](\d+(?:\.\d+)?).+fixed'
        re = self.re.search(_re, chapter, self.re.I)
        if re:
            return re.group(1)
        return chapter

    def get_content(self):
        url = self.get_url()
        if self.__local_storage:
            url = self.__local_storage
        return self.http_get(url)

    def get_manga_name(self) -> str:
        selector = '.main-content-inner .page-header'
        header = self.html_fromstring(self.get_url(), selector, 0)
        link = header.cssselect('a.active + a')
        if link:
            link = self.http().normalize_uri(link.get('href'))
            self.__local_storage = link
            header = self.html_fromstring(link, selector, 0)
        return header.text_content().strip().replace('/', '_')  # http://www.mangasail.com/content/12-prince-manga

    def _fix_chapters(self, items):
        # http://www.mangasail.com/content/go-toubun-no-hanayome-30-%E2%80%93-fixed
        # I wanted to sleep. Maybe fix it someday.
        found = []
        result = []
        n = self.http().normalize_uri
        for i in items:
            name, url = i.text_content().strip(), i.get('href')
            _name = self._parse_ch(name)
            if ~url.find('-fixed'):
                found.append(_name)
        for i in items:
            name = i.text_content().strip()
            _name = self._parse_ch(name)
            url = i.get('href')
            if _name not in found or ~url.find('-fixed'):
                result.append((name, n(url)))
        return result

    def get_chapters(self):
        items = self.document_fromstring(self.content, '.chlist td a')
        items = self._fix_chapters(items)
        return sorted(items, key=lambda n: float(self._parse_ch(n[0])), reverse=True)

    def prepare_cookies(self):
        self._base_cookies()

    def get_files(self):
        url = self.chapter[1]
        items = self.html_fromstring('{}{}'.format(url, '?page=all'), '#images img')
        n = self.http().normalize_uri
        return [n(i.get('src')) for i in items]

    def get_cover(self) -> str:  # TODO
        cover = self.document_fromstring(self.content, 'iframe.authcache-esi-assembly', 0)
        cover = self.json.loads(cover.text_content().strip()).get('field')
        key = cover.keys()[0]
        cover = self.document_fromstring(cover.get(key), '.field-type-image img', 0)
        return self.http().normalize_uri(cover.get('src'))

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.chapter[1]


main = MangaTailCom



from manga_py.provider import Provider
from .helpers.std import Std


class ReaderIMangaScansOrg(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        re = self.re.search(r'://.+?/[^/]+/([^/]+)', self.chapter)
        return re.group(1)

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'://.+?/([^/]+)')

    def get_chapters(self):
        sel = '.subnav-bind-top > .controls > div > .dropdown-menu li a'
        return self._elements(sel)

    def get_files(self):
        try:
            content = self.http_get(self.chapter)
            items = self.re.search(r'var\s+pages\s*=\s*(\[.+\])', content)
            items = self.json.loads(items.group(1))
            url = items[0]
            del items[0]
            return ['{}/{}{}'.format(self.domain, url, i) for i in items]
        except Exception:
            return []

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReaderIMangaScansOrg



from manga_py.provider import Provider
from .helpers import tsumino_com
from .helpers.std import Std


class TsuminoCom(Provider, Std):
    __local_storage = None

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_content(self):
        url = self.get_url()
        if ~url.find('/Read/'):
            url = self.html_fromstring(url, '#backToIndex + a', 0).get('href')
        return self.http_get(self.http().normalize_uri(url))

    def get_manga_name(self) -> str:
        return 'tsumino_' + self.re.search(r'/(?:Info|View)/(\d+)', self.get_url()).group(1)

    def get_chapters(self):
        return [b'']

    def prepare_cookies(self):
        self._base_cookies()

    def get_files(self):
        idx = self.re.search(r'/(?:Info|View)/(\d+)', self.get_url()).group(1)
        test_url = '{}/Read/View{}'.format(self.domain, idx)
        if ~self.http_get(test_url).find('/recaptcha'):
            cookies = tsumino_com.TsuminoCom(self).get_cookies(test_url).get_dict()
            for i in cookies:
                self._storage['cookies'][i] = cookies[i]

        with self.http().post(
            '{}/Read/Load'.format(self.domain, idx),
            headers={
                'X-Requested-With': 'XMLHttpRequest',
                'Referer': '{}/Read/View/{}'.format(self.domain, idx),
                'Pragma': 'no-cache',
                'Cache-Control': 'no-cache',
                'Accept-Language': 'en-US;q=0.8,en;q=0.7',
            },
            data={'q': idx}
        ) as resp:
            items = resp.json().get('reader_page_urls', [])
        d = str(self.domain)
        return [d + '/Image/Object?name=' + i for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('img.book-page-image')

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.get_url()


main = TsuminoCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaFoxCom(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/chapter-(\d+(?:-\d+)?)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.(?:com|io)/([^/]+)')

    def get_chapters(self):
        return self._elements('.list_chapter a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#list_chapter img')

    def get_cover(self) -> str:
        return self._cover_from_content('.info_image img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaFoxCom



from manga_py.provider import Provider
from .helpers.std import Std


class ReadEgScansCom(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/Chapter_(\d+)(.*)', self.chapter)
        return self._join_groups(idx.groups())

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.com/([^/]+)')

    def get_chapters(self):
        parser = self.document_fromstring(self.content)
        items = self._first_select_options(parser, 'select[name="chapter"]', False)
        url = '%s/%s/{}' % (self.domain, self.manga_name)
        return [url.format(i.get('value')) for i in items[::-1]]

    def get_files(self):
        url = self.chapter
        content = self.http_get(url)
        items = self.re.findall(r'img_url\.push\s?\(\s?\'(.+)\'\s?\)', content)
        domain = self.domain
        return ['{}/{}'.format(domain, i) for i in items]

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadEgScansCom



from manga_py.provider import Provider
from .helpers.std import Std
from .helpers.manganelo_com_helper import check_alternative_server


class MangaNeloCom(Provider, Std):
    chapter_re = r'[/-]chap(?:ter)?[_-](\d+(?:\.\d+)?)'
    _prefix = '/manga/'
    __alternative_cdn = 'https://bu2.mkklcdnbuv1.com'

    def get_chapter_index(self) -> str:
        return self.re.search(self.chapter_re, self.get_chapter())\
            .group(1).replace('.', '-')

    def get_content(self):
        return self._get_content('{}%s{}' % self._prefix)

    def get_manga_name(self) -> str:
        return self._get_name('/(?:manga|chapter)/([^/]+)')

    def get_chapters(self):
        return self._elements('.panel-story-chapter-list a')

    def get_files(self):
        chapter = self.get_chapter()

        parser = self.html_fromstring(chapter)
        images = self._images_helper(parser, '.container-chapter-reader img')
        return check_alternative_server(images, self.__alternative_cdn, headers={
            'Referer': chapter,
            'Accept': 'image/webp,*/*',
        })

    def get_cover(self) -> str:
        return self._cover_from_content('.manga-info-pic img')

    def get_chapter(self):
        return self.chapter


main = MangaNeloCom



import json
from concurrent.futures import ThreadPoolExecutor
from logging import info, warning, error
from sys import stderr

from PIL import Image

from .base_classes import Archive
from .base_classes.comic_info_builder import Page, ComicInfo
from .fs import (
    is_file,
    file_size,
)
from .meta import repo_url, version


class BaseDownloadMethod(object):
    def __init__(self, provider):
        self.provider = provider
        self.log = provider.log
        self.http = self.provider.http
        self.chapter_progress = self.provider.chapter_progress
        self.global_progress = self.provider.global_progress
        self.volume = "0"

    def download_chapter(self, url, path):
        pass

    def already_downloaded(self):
        # check
        _path = '%s.%s' % self.provider.get_archive_path()
        not_allow_archive = not self.provider._params.get('rewrite_exists_archives', False)

        return not_allow_archive and is_file(_path)


class OnePerOneDownloader(BaseDownloadMethod):
    chapter_url = None
    path = None
    _archive = None

    def __init__(self, provider):
        super().__init__(provider)
        self.__pages_cache = []

    def download_chapter(self, chapter_url, path):
        self.chapter_url = chapter_url
        self.path = path
        try:
            self.files = self.provider.get_files()
            self._loop_files()
        except Exception as e:
            # Main debug here
            if self.provider._debug:
                raise e
            self.log([e], file=stderr)
            self.provider._info.set_last_volume_error(e)

    def _loop_files(self):
        if isinstance(self.files, list):
            info('Processing {} files'.format(len(self.files)))

            if len(self.files) == 0:
                # see Std
                error('Error processing file: %s' % self.provider.get_archive_name())
                return

            self._archive = Archive()
            self._archive.not_change_files_extension = self.provider._params.get('not_change_files_extension', False)
            self._archive.no_webp = self.provider._image_params.get('no_webp', False)

            if self.provider._save_chapter_info:
                self._archive.write_file('info.json', json.dumps(self.chapter_url))

            chapter_info = self.provider.chapter_details(self.chapter_url)
            if self.provider._save_chapter_info:
                if chapter_info is not None:
                    self._archive.write_file('eduhoribe.json', json.dumps(chapter_info))
                else:
                    warning('No chapter details was found!')
                    warning('Possibly the provider has not yet been implemented to get this information')

            self.chapter_progress(len(self.files), 0, True)

            self._multi_thread_save(self.files)

            self._make_archive()
        else:
            error('Bad files list type')

    def _make_archive(self):
        # """
        # make book info
        # """
        if self.provider._params['cbz']:
            _info = self._book_info_xml()
            self._archive.write_file('ComicInfo.xml', str(_info))

        info = 'Site: {}\nDownloader: {}\nVersion: {}'.format(self.provider.get_url(), repo_url, version)

        self._archive.write_file('info.txt', info)

        if self._archive.has_error:
            full_path = '%s.IMAGES_SKIP_ERROR.%s' % self.path
            self.provider._info.set_last_volume_error(str(self._archive.error_list))
            if self.provider._skip_incomplete_chapters:
                warning("Skipping incomplete chapter: %s.%s" % self.path)
                return
        else:
            full_path = '%s.%s' % self.path

        try:
            self._archive.make(full_path)
        except OSError as e:
            error(e)
            self.provider._info.set_last_volume_error(str(e))
            raise e

    def _multi_thread_save(self, files):
        max_threads = int(self.provider._params.get('max_threads', 0))
        max_threads = min(max(max_threads, 1), 10)

        with ThreadPoolExecutor(max_workers=max_threads) as executor:
            m = executor.map(self.save_file, *zip(*enumerate(files)))
            for i, p in enumerate(m, start=1):
                self.chapter_progress(len(self.files), i)

    def save_file(self, idx=None, url=None, callback=None, in_arc_name=None):
        _path, idx, _url = self.provider.before_download_file(idx, url)

        if _path is None:
            return

        if not is_file(_path) or file_size(_path) < 32:
            self.http().download_file(_url, _path, idx)

        _path, _in_arc_name = self.provider.after_file_save(_path, idx)

        if _path is None:
            info('after_file_save is None. Idx: {} / Url: {}'.format(idx, url), file=stderr)
            return None

        self._archive.add_file(_path, in_arc_name=(_in_arc_name or in_arc_name))
        callable(callback) and callback()

        if self.provider._params['cbz']:
            with Image.open(_path) as r:  # type: Image.Image
                w, h = r.size

            self.__pages_cache.append(Page(
                index=idx,
                size=file_size(_path),
                width=w,
                height=h,
            ))

        return _path

    def _book_info_xml(self):
        comic_info = ComicInfo()

        comic_info.volume(self.provider.get_chapter_index())
        comic_info.title(self.provider.name)
        comic_info.pages(self.__pages_cache)
        comic_info.page_count(str(len(self.__pages_cache)))

        return comic_info


class WholeArchiveDownloader(BaseDownloadMethod):
    def download_chapter(self, chapter_url, path):
        self.provider.chapter_progress(1, 0, True)
        try:
            self.provider.http().download_file(chapter_url, '%s.%s' % path)
        except Exception as e:
            if self.provider._debug:
                raise e
            self.log([e], file=stderr)
            self.provider._info.set_last_volume_error(e)
        self.provider.chapter_progress(1, 1)

        # if isinstance(self._storage['files'], list):
        #     info('Processing {} files'.format(len(self._storage['files'])))
        #
        # self.__images_cache = []
        #
        # # ///
        #
        # if not is_file(_path) or file_size(_path) < 32:
        #     self.http().download_file(_url, _path, idx)
        #
        # self.after_file_save(_path, idx)
        # self._archive.add_file(_path)
        #
        # with Image.open(_path) as r:  # type: Image.Image
        #     w, h = r.size
        #
        # self.__images_cache.append(Page(
        #     index=idx,
        #     size=file_size(_path),
        #     width=w,
        #     height=h,
        # ))
        #
        # callable(callback) and callback()

    def make_archive(self):
        _path = self.get_archive_path()

        info = 'Site: {}\nDownloader: {}\nVersion: {}'.format(self.get_url(), repo_url, version)

        # """
        # make book info
        # """
        # if self._params['cbz']:
        #     self._archive.add_book_info(self._arc_meta_info())

        self._archive.write_file('info.txt', info)

        if 'cbz' in self._params and self._params['cbz']:
            book_info = ComicInfo()
            book_info.title(self.get_manga_name()) # todo: normal title maybe?
            book_info.pages(self.__images_cache)

            self._archive.write_file('ComicInfo.xml', str(book_info))

        if self._archive.has_error:
            full_path = '%s.IMAGES_SKIP_ERROR.%s' % _path
            self._info.set_last_volume_error(str(self._archive.error_list))
            if self._skip_incomplete_chapters:
                warning("Skipping incomplete chapter: %s.%s" % _path)
                return
        else:
            full_path = '%s.%s' % _path

        try:
            self._archive.make(full_path)
        except OSError as e:
            error(e)
            self._info.set_last_volume_error(str(e))
            raise e




from manga_py.provider import Provider
from .helpers.std import Std


class MangaReaderNet(Provider, Std):

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        return self.re.search(r'\.\w{2,7}/[^/]+/([^/]+)', chapter).group(1)

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        return self._elements('#listing a')[::-1]

    @staticmethod
    def _get_img(parser):
        return [i.get('src') for i in parser.cssselect('#img')]

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        pages = self._first_select_options(parser, 'select#pageMenu')
        images = self._get_img(parser)
        for i in pages:
            parser = self.html_fromstring(self.domain + i.get('value'))
            images += self._get_img(parser)
        return images

    def get_cover(self):
        return self._cover_from_content('#mangaimg img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaReaderNet



import re
from manga_py.provider import Provider
from .helpers.std import Std
from html import escape


class MangaDexOrg(Provider, Std):
    __content = None
    __chapters = None
    __languages = None
    __countries = {
        '': 'Other',
        'bd': 'Bengali',
        'bg': 'Bulgarian',
        'br': 'Portuguese (Br)',
        'cn': 'Chinese (Simp)',
        'ct': 'Catalan',
        'cz': 'Czech',
        'de': 'German',
        'dk': 'Danish',
        'es': 'Spanish (Es)',
        'fi': 'Finnish',
        'fr': 'French',
        'gb': 'English',
        'gr': 'Greek',
        'hk': 'Chinese (Trad)',
        'hu': 'Hungarian',
        'id': 'Indonesian',
        'il': 'Hebrew',
        'in': 'Hindi',
        'ir': 'Persian',
        'it': 'Italian',
        'jp': 'Japanese',
        'kr': 'Korean',
        'lt': 'Lithuanian',
        'mm': 'Burmese',
        'mn': 'Mongolian',
        'mx': 'Spanish (LATAM)',
        'my': 'Malay',
        'nl': 'Dutch',
        'no': 'Norwegian',
        'ph': 'Filipino',
        'pl': 'Polish',
        'pt': 'Portuguese (Pt)',
        'ro': 'Romanian',
        'rs': 'Serbo-Croatian',
        'ru': 'Russian',
        'sa': 'Arabic',
        'se': 'Swedish',
        'th': 'Thai',
        'tr': 'Turkish',
        'ua': 'Ukrainian',
        'vn': 'Vietnamese',
    }

    def _get(self, part):
        return self.http().requests('{}/api/v2/{}'.format(
            self.domain,
            part.format(self.manga_idx())),
        ).json()

    def get_archive_name(self) -> str:
        prev = super().get_archive_name()
        code = self.chapter['language']
        return '{}-{}'.format(prev, self.__countries.get(code, 'Other'))

    def get_chapter_index(self) -> str:
        return self.chapter['chapter'].replace('.', '-')

    def manga_idx(self):
        return self.re.search(r'/(?:manga|title)/(\d+)', self.get_url()).group(1)

    def get_content(self):
        return 'nope'

    def get_manga_name(self) -> str:
        self.__content = self._get('manga/{}').get('data', {})
        return self.__content.get('title')

    def get_chapters(self):
        _ch = self._chapters

        if len(self._languages) > 1:
            languages = self._quest_languages()

            _ch = self.filter_chapters(_ch, languages)

        translator = self.arg('translator')
        if translator is not None:
            _ch = self.filter_chapters_translator(_ch, translator)

        return _ch

    def get_files(self):
        content = self._get(f'chapter/{self.chapter["hash"]}').get('data', {})
        server = content['server']
        _hash = content['hash']
        return [f'{server}{_hash}/{img}' for img in content['pages']]

    def get_cover(self) -> str:
        return self.content['mainCover']

    def chapter_for_json(self) -> str:
        return '{}-{}'.format(self.chapter['volume'] or '0', self.chapter['chapter'])

    @property
    def _chapters(self):
        if self.__chapters is None:
            self.__chapters = self._get('manga/{}/chapters').get('data', {})
        return self.__chapters.get('chapters', [])

    def _quest_languages(self):
        arg_language = self.arg('language')
        if arg_language is None:
            languages = self.quest(
                [],
                'Available languages:\n{}\n\n'
                'Please, select your lang (empty for all, comma for delimiter lang):'.format(
                    '\n'.join(self._languages)
                ))
        else:
            languages = arg_language

        return list([lng.strip() for lng in languages.split(',')])

    @property
    def _languages(self) -> list:
        if self.__languages is None:
            self.__languages = list(set([ch['language'] for ch in self._chapters]))
        return self.__languages

    def filter_chapters(self, chapters, languages: list) -> list:
        if len(languages) == 0 or languages[0] == '':
            return chapters
        return [chapter for chapter in chapters if chapter['language'] in languages]

    def filter_chapters_translator(self, chapters, translator: str) -> list:
        enc_translator = escape(translator)
        return [chapter for chapter in chapters if len(set(self._translators(chapter)) & {enc_translator}) > 0]

    def _translators(self, chapter):
        groups = self.__chapters.get('groups', [])
        return [g['name'] for g in groups if g['id'] in chapter['groups']]

    # region specified data for eduhoribe/comic-builder

    def chapter_details(self, chapter) -> dict:
        return {
            'chapter': chapter['chapter'],
            'volume': chapter['volume'],
            'title': chapter['title'],
            'language': chapter['language'],
            'publisher': 'See "publishers"',
            'publishers': self._translators(chapter)
        }

    @staticmethod
    def _flat_array(arg):
        if arg is None:
            return ['']
        if type(arg) == list:
            return arg
        if type(arg) == str:
            return [arg]
        raise TypeError('Unknown type!')

    def manga_details(self):
        author = self._flat_array(self.__content.get('author', ''))
        artist = self._flat_array(self.__content.get('artist', ''))
        return {
            'id': self.manga_idx(),
            'title': self.__content['title'],
            'description': self.__content['description'],
            'authors': [author for author in {*author, *artist} if author != ''],
            'sauce': self.original_url,
            'covers': {'main': self.__content.get('mainCover')}
        }
    # endregion


main = MangaDexOrg



from manga_py.provider import Provider
from .helpers.std import Std


class SeriMangaCom(Provider, Std):
    def get_chapter_index(self) -> str:
        idx = self.chapter[1].split('.')
        return '-'.join([
            idx[0],
            idx[1] if len(idx) > 1 else '0',
        ])

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/([-\w]+)')

    def get_chapters(self):
        items = self._elements('li.spl-list-item a')
        n = self.http().normalize_uri
        return [(n(i.get('href')), i.cssselect('span')[0].text_content().strip()) for i in items]

    def get_files(self):
        parser = self.html_fromstring(self.chapter[0])
        images = self._images_helper(parser, '.chapter-pages > img', 'data-src', 'src')
        n = self.http().normalize_uri
        return [n(image) for image in images]

    def get_cover(self) -> str:
        return self.parse_background(self._elements('.seri-img')[0])

    def chapter_for_json(self) -> str:
        return self.chapter[0]


main = SeriMangaCom



from manga_py.provider import Provider


class EightMusesCom:
    provider = None

    def __init__(self, provider: Provider):
        self.provider = provider
        self._n = provider.http().normalize_uri

    def is_images_page(self, parser) -> bool:
        if not parser:
            return False
        return self.provider.re.search(r'/\d+$', parser[0].get('href')) is not None

    def parser(self, url, selector):
        return self.provider.html_fromstring(self._n(url), selector)

    def chapters(self, parser) -> list:
        if self.is_images_page(parser):
            return [parser]
        items = []
        selector = self.provider.chapter_selector
        for i in parser:
            items += self.chapters(self.parser(i.get('href'), selector))
        return items



from manga_py.provider import Provider
from .helpers.std import Std


class SoMangaNet(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        re = self.re.compile('/leitor/[^/]+/([^/]+)')
        return re.search(self.chapter).group(1)

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/[^/]+/([^/]+)')

    def get_chapters(self):
        return self._elements('ul.capitulos li > a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, 'img.img-manga')

    def get_cover(self):
        return self._cover_from_content('.manga .col-sm-4 .img-responsive')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = SoMangaNet



from manga_py.provider import Provider
from .helpers.std import Std


class MangaLibMe(Provider, Std):

    def get_chapter_index(self) -> str:
        return '{}_{}'.format(
            self.chapter_for_json(),
            self.chapter['chapter_name']
        )

    def get_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,7}/([^/]+)')

    def get_chapters(self):
        _json = self.re.search(r'__DATA__\s*=\s*(\{.+?\});', self.content)

        if _json is None:
            return []

        return self.json.loads(_json.group(1)).get('chapters', {}).get('list', [])

    def get_files(self):
        content = self.http_get(self.chapter)
        images = self.re.search(r'__pg\s*=\s*(\[.+\])', content).group(1)
        info = self.re.search(r'__info\s*=\s*(\{.+\})', content).group(1)
        images = self.json.loads(images)
        info = self.json.loads(info)
        _manga = info['img']['url']
        _s = info['servers']
        _server = _s.get('main', _s.get('secondary'))

        return ['{}{}{}'.format(_server, _manga, i['u']) for i in images]

    def get_cover(self):
        return self._cover_from_content('.media-sidebar__cover > img')

    def prepare_cookies(self):
        self.cf_scrape(self.get_url())

    def chapter_for_json(self) -> str:
        return '{}-{}'.format(
            self.chapter['chapter_volume'] or '',
            self.chapter['chapter_number'],
        ).rstrip('-')


main = MangaLibMe



from manga_py.provider import Provider
from .helpers.std import Std


class _Template(Provider, Std):
    def get_chapter_index(self) -> str:
        return self.re.search(r'-chapter-(\d+(?:\.\d+)?)', self.chapter).group(1).replace('.', '-')

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/([\w-]+)')

    def get_chapters(self):
        return self._elements('ul a[href*="/chapter/"]')

    def get_files(self):
        parser = self.html_fromstring(self.chapter, '#arraydata', 0)
        content = self.element_text_content(parser)

        if content is None:
            return []

        return content.split(',')

    def get_cover(self) -> str:
        return self._cover_from_content('.imgdesc > img')


main = _Template



from logging import warning, error
from os import path
from zipfile import ZipFile, ZIP_DEFLATED

from manga_py.fs import is_file, make_dirs, basename, dirname, unlink, get_temp_path
from manga_py.manga_image import MangaImage


class Archive:
    _archive = None
    _writes = None
    files = None
    not_change_files_extension = False
    no_webp = False
    error_list = []

    @property
    def has_error(self):
        return self.error_list != []

    def __init__(self):
        self.files = []
        self._writes = {}

    def write_file(self, in_arc_name, data):
        self._writes[in_arc_name] = data

    def add_file(self, file, in_arc_name=None):
        if in_arc_name is None:
            in_arc_name = basename(file)
        if self.__test_is_image(file):
            self.files.append((file, in_arc_name))

    def __add_files(self):
        for file in self.files:
            if is_file(file[0]):
                file = self._file(file)
                self._archive.write(*file)
            else:
                warning('"%s" it\'s not file! Skip' % file[0])

    def _file(self, file):
        ext = self.__update_image_extension(file[0])
        if self.no_webp and ext[ext.rfind('.'):] == '.webp':
            jpeg = ext[:ext.rfind('.')] + '.jpeg'
            jpeg_path = path.join(dirname(file[0]), jpeg)
            MangaImage(file[0]).convert(jpeg_path)
            file = jpeg_path, jpeg
        elif ext:
            file = file[0], ext
        return file

    def __test_is_image(self, _path):
        if not MangaImage.is_image(_path):
            self.error_list.append('File "%s" isn\'t image' % _path)
            warning('File "%s" isn\'t image' % _path)
            return False
        return True

    def __add_writes(self):
        for file in self._writes:
            self._archive.writestr(file, self._writes[file])

    def make(self, dst: str):
        if not len(self.files):
            error('Files list empty. Skip making archive')
            return

        if self.has_error:
            warning('Archive %s have missed files' % dst)
            self.error_list = []

        make_dirs(dirname(dst))

        self._archive = ZipFile(dst, 'w', ZIP_DEFLATED)
        try:
            self.__add_files()
            self.__add_writes()
            self._archive.close()
        except OSError as e:
            self._archive.close()
            raise e
        self._maked()

    def _maked(self):
        for file in self.files:
            try:
                unlink(file[0])
            except Exception:
                error('File %s can\'t deleted' % file[0])

    def __update_image_extension(self, filename) -> str:
        fn, extension = path.splitext(filename)
        if not self.not_change_files_extension:
            ext = MangaImage.real_extension(get_temp_path(filename))
            if ext:
                extension = ext
        return basename(fn + extension)



import sys
from argparse import ArgumentParser
from getpass import getpass
from logging import error, warning
from os import name as os_name
from typing import Union

from progressbar import ProgressBar

from manga_py.fs import check_free_space, get_temp_path
from manga_py.parser import Parser


class Cli:  # pragma: no cover
    args = None
    parser = None
    _info = None
    __progress_bar = None

    def __init__(self, args: Union[dict, ArgumentParser], info=None):
        if type(args) == ArgumentParser:
            warning('args type of ArgumentParser deprecated. Please, use dict istread')
            args = args.parse_args().__dict__

        self.args = args
        self.parser = Parser(args)
        self._info = info

        space = self.args['min_free_space']
        if not check_free_space(get_temp_path(), space) or not check_free_space(self.args['destination'], space):
            raise OSError('No space left on device')

    def start(self):
        try:
            self.parser.init_provider(
                chapter_progress=self.chapter_progress,
                global_progress=self.global_progress,
                log=self.print,
                quest=self.quest,
                quest_password=self.quest_password,
                info=self._info,
            )
        except AttributeError as e:
            error('\n'.join([
                'Please check if your inputed domain is supported by manga-py: ',
                '- https://manga-py.com/manga-py/#resources-list',
                '- https://manga-py.github.io/manga-py/#resources-list (alternative)',
                'Make sure that your inputed URL is correct',
                'Trace:',
            ]))
            raise e

        self.parser.start()
        self.__progress_bar and self.__progress_bar.value > 0 and self.__progress_bar.finish()
        self.args['quiet'] or self.print(' ')

    def __init_progress(self, items_count: int, re_init: bool):
        if re_init or not self.__progress_bar:
            if re_init:
                self.__progress_bar.finish()

            bar = ProgressBar()
            self.__progress_bar = bar(range(items_count))
            self.__progress_bar.init()

    def chapter_progress(self, items_count: int, current_item: int, re_init: bool = False):
        if not items_count \
                or self.args['no_progress'] \
                or self.args['global_progress'] \
                or self.args['quiet'] \
                or self.args['print_json'] \
                or self.args['debug']:
            return

        self._progress(items_count, current_item, re_init)

    def global_progress(self, items_count: int, current_item: int, re_init: bool = False):
        if not items_count \
                or self.args['no_progress'] \
                or not self.args['global_progress'] \
                or self.args['quiet'] \
                or self.args['print_json'] \
                or self.args['debug']:

            return
        self._progress(items_count, current_item, re_init)

    def _progress(self, items_count: int, current_item: int, re_init: bool = False):
        current_val = 0

        if self.__progress_bar:
            current_val = self.__progress_bar.value

        self.__init_progress(items_count, re_init and current_val > 0)
        self.__progress_bar.update(current_item)

    def print(self, text, **kwargs):

        if os_name == 'nt':
            text = str(text).encode().decode(sys.stdout.encoding, 'ignore')

        self.args['quiet'] or print(text, **kwargs)

    def _single_quest(self, variants, title):
        print(title)

        for v in variants:
            print(v)

        return input()

    def _multiple_quest(self, variants, title):
        print(f'Accept - blank line + enter\n{title}')

        for v in variants:
            print(v)

        result = []

        while True:
            _ = input().strip()

            if not len(_):
                return result

            result.append(_)

    def quest(self, variants: enumerate, title: str, select_type=0):  # 0 = single, 1 = multiple

        if select_type:
            return self._multiple_quest(variants, title)

        return self._single_quest(variants, title)

    def quest_password(self, title):
        return getpass(title)



from typing import NamedTuple, List, Optional

__all__ = ['Page', 'ComicInfo']


class Page(NamedTuple):
    index: Optional[int]
    size: Optional[int]
    width: Optional[int]
    height: Optional[int]


keys = {
    'title': 'Title',
    'series': 'Series',
    'number': 'Number',
    'volume': 'Volume',
    'alternate_series': 'AlternateSeries',
    'series_group': 'SeriesGroup',
    'summary': 'Summary',
    'notes': 'Notes',
    'year': 'Year',
    'month': 'Month',
    'day': 'Day',
    'writer': 'Writer',
    'penciller': 'Penciller',
    'inker': 'Inker',
    'colorist': 'Colorist',
    'letterer': 'Letterer',
    'cover_artist': 'CoverArtist',
    'editor': 'Editor',
    'publisher': 'Publisher',
    'genre': 'Genre',
    'web': 'Web',
    'page_count': 'PageCount',
    'language': 'Language',
    'iso': 'ISO',
    'age_rating': 'AgeRating',
    'characters': 'Characters',
    'teams': 'Teams',
    'scan_information': 'ScanInformation',
}


class ComicInfo:
    __slots__ = ('__data', '__pages', '_keys')

    def __init__(self):
        self.__data = {
            'title': None,
            'series': None,
            'number': None,
            'volume': None,
            'alternate_series': None,
            'series_group': None,
            'summary': None,
            'notes': None,
            'year': None,
            'month': None,
            'day': None,
            'writer': None,
            'penciller': None,
            'inker': None,
            'colorist': None,
            'letterer': None,
            'cover_artist': None,
            'editor': None,
            'publisher': None,
            'genre': None,
            'web': None,
            'page_count': None,
            'language': None,
            'iso': None,
            'age_rating': None,
            'characters': None,
            'teams': None,
            'scan_information': None,
        }

        self.__pages = []

    def __str__(self):
        schema = 'http://www.w3.org/2001/XMLSchema'
        lines = [
            '<?xml version="1.0"?>',
            f'<ComicInfo xmlns:xsd="{schema}" xmlns:xsi="{schema}-instance">'
        ]
        for key in self.__data.keys():
            if self.__data[key] is None:
                continue

            index = keys[key]
            value = self.__data[key]

            lines.append(f'  <{index}>{value}</{index}>')

        lines.append('    <Pages>')
        pages = sorted(self.__pages, key=lambda p: (p.index or 0))
        for idx, page in enumerate(pages):  # type: int, Page
            _data = ''
            if page.index == 0:
                _data += ' Type="FrontCover"'

            if page.width is not None and page.height is not None:
                _data += f' ImageWidth="{page.width}" ImageHeight="{page.height}"'

            size = page.size or 0

            lines.append(f'    <Page Image="{idx}" ImageSize="{size}"{_data}/>')
        lines.append('    </Pages>')
        lines.append('</ComicInfo>')

        return '\n'.join(lines)

    def title(self, value: str):
        self.__data['title'] = value

    def series(self, value: str):
        self.__data['series'] = value

    def number(self, value: str):
        self.__data['number'] = value

    def volume(self, value: str):
        self.__data['volume'] = value

    def alternate_series(self, value: str):
        self.__data['alternate_series'] = value

    def series_group(self, value: str):
        self.__data['series_group'] = value

    def summary(self, value: str):
        self.__data['summary'] = value

    def notes(self, value: str):
        self.__data['notes'] = value

    def year(self, value: str):
        self.__data['year'] = value

    def month(self, value: str):
        self.__data['month'] = value

    def day(self, value: str):
        self.__data['day'] = value

    def writer(self, value: str):
        self.__data['writer'] = value

    def penciller(self, value: str):
        self.__data['penciller'] = value

    def inker(self, value: str):
        self.__data['inker'] = value

    def colorist(self, value: str):
        self.__data['colorist'] = value

    def letterer(self, value: str):
        self.__data['letterer'] = value

    def cover_artist(self, value: str):
        self.__data['cover_artist'] = value

    def editor(self, value: str):
        self.__data['editor'] = value

    def publisher(self, value: str):
        self.__data['publisher'] = value

    def genre(self, value: str):
        self.__data['genre'] = value

    def web(self, value: str):
        self.__data['web'] = value

    def page_count(self, value: str):
        self.__data['page_count'] = value

    def language(self, value: str):
        self.__data['language'] = value

    def iso(self, value: str):
        self.__data['iso'] = value

    def age_rating(self, value: str):
        self.__data['age_rating'] = value

    def characters(self, value: str):
        self.__data['characters'] = value

    def teams(self, value: str):
        self.__data['teams'] = value

    def scan_information(self, value: str):
        self.__data['scan_information'] = value

    def pages(self, pages: List[Page]):
        self.__pages = pages




from .helpers.std import Std
from .mangaonline_today import MangaOnlineToday


class AuthroneCom(MangaOnlineToday, Std):
    _ch_selector = '.mng_det ul.lst > li > a'

    def get_archive_name(self) -> str:
        idx = self.re.search('/manga/[^/]+/([^/]+/[^/]+)', self.chapter).group(1).split('.', 2)
        if len(idx) > 1:
            return 'vol_{:0>3}-{}_{}'.format(idx[0], *idx[1].split('/'))
        return 'vol_{:0>3}-0-{}'.format(*idx[0].split('/'))

    def get_chapter_index(self) -> str:
        return self.re.search('/manga/[^/]+/([^/]+)', self.chapter).group(1)

    def get_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):  # need sorting chapters: /manga/love_stage/
        items = self._elements(self._ch_selector)
        pages = self._elements('ul.lst + ul.pgg li:last-child > a')
        patern = r'list/(\d+)/'
        if pages and len(pages):
            link = pages[-1].get('href')
            page = self.re.search(patern, link).group(1)
            for i in range(2, int(page) + 1):
                page_link = self.re.sub(patern, 'list/%d/' % i, link)
                items += self._elements(self._ch_selector, self.http_get(page_link))
        return items

    def get_cover(self) -> str:
        return self._cover_from_content('#sct_content img.cvr')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = AuthroneCom



import random
import time

from manga_py.crypt.base_lib import BaseLib
from .request import Request


class GoogleDCP:
    host = 'proxy.googlezip.net'
    authkey = 'ac4500dd3b7579186c1b0620614fdb1f7d61f944'
    http = None

    def __init__(self, http: Request):
        self.http = http

    def randint(self):
        return random.randint(0, 999999999)

    def _build_header(self):
        timestamp = int(time.time())
        md5 = BaseLib.md5('{}{}{}'.format(timestamp, self.authkey, timestamp))
        return 'Chrome-Proxy: ps={}-{}-{}-{}, sid={}, c=win, b=3029, p=110'.format(
            int(time.time()),
            self.randint(),
            self.randint(),
            self.randint(),
            BaseLib.str2hex(md5.hexdigest())
        )

    def set_proxy(self):
        self.http.proxies['http'] = self.host
        self.http.headers = self._build_header()
        return self.http



from .manganelo_com import MangaNeloCom


class MangaBatCom(MangaNeloCom):
    def get_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self.text_content(self.content, '.story-info-right h1')

    def prepare_cookies(self):
        if not ~self.get_url().find(self._prefix):
            self._prefix = '/'

        super().prepare_cookies()


main = MangaBatCom



from .rawdevart_com_old import RawDevArtComOld


class ISekaiScanCom(RawDevArtComOld):
    _chapter_selector = r'/(?:chapter-)?(\d+(?:[\a-zA-Z\.-]*?\d+)?(?:-\w+)?)/'

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.page-break img.wp-manga-chapter-img', 'data-src')


main = ISekaiScanCom



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from sys import stderr

from setuptools import setup, find_packages


author = 'Sergey Zharkov'
license_type = 'MIT'
email = 'sttv-pc@mail.ru'
version = '1.29.1'
repo_url = 'https://github.com/manga-py/manga-py'


REQUIREMENTS = ['cloudscraper', 'cssselect>=1.1', 'lxml~=4.6', 'packaging>=20.3', 'Pillow~=8.1', 'progressbar2>=3.50', 'pycryptodome>=3.9', 'PyExecJS>=1.5', 'requests>=2.23', 'better_exceptions>=0.2', 'pysocks>=1.7', '# aiocfscrape  # cf', ]

try:
    import lxml
    REQUIREMENTS = [r for r in REQUIREMENTS if not r.startswith('lxml')]
except ModuleNotFoundError:
    pass

try:
    from PIL import Image
    REQUIREMENTS = [r for r in REQUIREMENTS if not r.startswith('Pillow')]
except ModuleNotFoundError:
    pass


long_description = """
Universal manga downloader.

Please see https://github.com/manga-py/manga-py
"""

release_status = 'Development Status :: 5 - Production/Stable'
if ~version.find('beta'):
    release_status = 'Development Status :: 4 - Beta'
if ~version.find('alpha'):
    release_status = 'Development Status :: 3 - Alpha'


setup(
    name='manga_py',
    packages=find_packages(exclude=(
        'tests',
        '.github',
        'Manga',
        'helpers',
        'mypy_cache',
    )),
    include_package_data=True,
    version=version,
    description='Universal assistant download manga.',
    long_description=long_description,
    author=author,
    author_email=email,
    url=repo_url,
    zip_safe=False,
    data_files=[
        ('manga_py/storage', [
            'manga_py/storage/.passwords.json.dist',
            'manga_py/storage/.proxy.txt',
            'manga_py/crypt/aes.js',
            'manga_py/crypt/aes_zp.js',
        ]),
    ],
    keywords=['manga-downloader', 'manga', 'manga-py'],
    license=license_type,
    classifiers=[  # look here https://pypi.python.org/pypi?%3Aaction=list_classifiers
        release_status,
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Environment :: Console',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet :: WWW/HTTP',
    ],
    python_requires='>=3.6.5',
    install_requires=REQUIREMENTS,
    entry_points={
        'console_scripts': [
            'manga-py = manga_py.util:main',
        ]
    }
)

print('\n'.join((
    '\n\nPlease remember that all sites earn on advertising.',
    'Remember to visit them from your browser.',
    'Thanks!\n'
)), file=stderr)


from manga_py.provider import Provider
from .helpers.std import Std


class MangaOnNet(Provider, Std):
    __has_ch = False

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        if self.__has_ch:
            var = {'vol': idx[0], 'ch': idx[1]}
        else:
            var = {'vol': idx}
        return self.normal_arc_name(var)

    def get_chapter_index(self) -> str:
        selector = r'(?:vol-?(\d+))?(?:-ch-?(\d+))'
        ch = self.chapter
        re = self.re.search(selector, ch)
        if re:
            self.__has_ch = True
            re = re.groups()
            return '{}-{}'.format(
                0 if not re[0] else re[0],
                re[1]
            )
        selector = r'.+-(\d+)'
        re = self.re.search(selector, ch)
        return '0-{}'.format(re.group(1))

    def get_content(self):
        url = '{}/manga-info/{}'.format(self.domain, self.manga_name)
        return self.http_get(url)

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('read-online'):
            url = self.html_fromstring(url, '.back-info a', 0).get('href')
        return self.re.search(r'/manga-info/([^/]+)', url).group(1)

    def get_chapters(self):
        return self._elements('.list-chapter li > a')

    def get_files(self):
        items = self.html_fromstring(self.chapter, '#list-img img')
        return [i.get('src') for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('.cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaOnNet



from manga_py.provider import Provider
from .helpers import tonarinoyj_jp
from .helpers.std import Std


class TonariNoYjJp(Provider, Std):
    helper = None

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_content(self):
        content = self._storage.get('main_content', None)
        if content is None:
            return self.http_get(self.get_url())
        return content

    def get_manga_name(self) -> str:
        h1 = self.document_fromstring(self.content, 'h1.series-header-title')
        if h1:
            return h1[0].text_content()
        return '__Manga__'

    def get_chapters(self):
        idx = self.re.search(r'/episode/(\d+)', self.get_url())
        items = self.helper.get_chapters(idx.group(1))
        return ['{}/episode/{}'.format(self.domain, i) for i in items]

    def get_files(self):
        doc = self.html_fromstring(self.chapter)
        images = []
        # img = doc.cssselect('.link-slot > img')  # sometimes 1x1 px
        # img and images.append(img[0].get('src'))
        images += [i.get('data-src') for i in doc.cssselect('img.js-page-image')]
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.link-slot > img')

    def prepare_cookies(self):
        self.helper = tonarinoyj_jp.TonariNoYjJp(self)

    def after_file_save(self, _path: str, idx: int):
        if idx:
            self.helper.solve_image(_path, idx)
        return super().after_file_save(_path, idx)

    def book_meta(self) -> dict:
        # todo meta
        pass


main = TonariNoYjJp



from re import compile
from requests import get
from requests.exceptions import HTTPError

RE = compile('^https?://[^/]+(/.+)$')


def check_alternative_server(images: list, other_cdn: str, **kwargs):
    if len(images):
        try:
            with get(images[0], stream=True, **kwargs) as image:
                image.raise_for_status()
        except HTTPError:
            return ['%s%s' % (other_cdn, RE.search(i).group(1)) for i in images]
    return images



from manga_py.provider import Provider
from .helpers.std import Std


class GoodMangaNet(Provider, Std):

    def get_chapter_index(self) -> str:
        return self.re.search(r'/chapter/(\d+)', self.chapter).group(1)

    def get_content(self):
        url = self.get_url()
        if ~url.find('/chapter/'):
            url = self.html_fromstring(url, '#manga_head h3 > a', 0).get('href')
        _id = self.re.search(r'net/(\d+/[^/]+)', url).group(1)
        return self.http_get('{}/{}'.format(self.domain, _id))

    def get_manga_name(self) -> str:
        url = self.get_url()
        reg = r'/([^/]+)/chapter/|net/\d+/([^/]+)'
        groups = self.re.search(reg, url).groups()
        return groups[0] if groups[0] else groups[1]

    @staticmethod
    def get_chapters_links(parser):
        return [i.get('href') for i in parser.cssselect('#chapters li > a')]

    def get_chapters(self):
        selector = '#chapters li > a'
        chapters = self._elements(selector)
        pagination = self._elements('.pagination li > button[href]')
        for i in pagination:
            chapters += self._elements(selector, self.http_get(i.get('href')))
        return chapters

    def get_files(self):
        img_selector = '#manga_viewer > a > img'
        parser = self.html_fromstring(self.chapter)
        images = self._images_helper(parser, img_selector)
        pages = self._first_select_options(parser, '#asset_2 select.page_select', True)
        for i in pages:
            _parser = self.html_fromstring(i.get('value'))
            images += self._images_helper(_parser, img_selector)
        return images

    def get_cover(self):
        pass  # TODO

    def book_meta(self) -> dict:
        # todo meta
        pass


main = GoodMangaNet



from manga_py.provider import Provider
from .helpers.std import Std


class MangaKatanaCom(Provider, Std):
    name_re = r'/manga/([^/]+)'

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/manga/.+?\d/c(\d+(?:\.\d)?(?:-v\d)?)')
        return re.search(self.chapter).group(1).replace('.', '-')

    def get_content(self):
        return self.http_get('{}/manga/{}'.format(
            self.domain, self._name()
        ))

    def get_manga_name(self) -> str:
        name = self._name()
        return name[:name.rindex('.')]

    def get_chapters(self):
        return self._elements('.chapters .chapter a')

    def get_files(self):
        content = self.http_get(self.chapter)
        items = self.re.search(
            r'var\s+\w+\s?=\s?(\[[\'"].+?[\'"]).?\]\s?;',
            content
        ).group(1).replace("'", '"') + ']'
        images = self.json.loads(items)
        return [img[::-1] for img in images]

    def get_cover(self) -> str:
        return self._cover_from_content('.cover img')

    def book_meta(self) -> dict:
        pass

    def _name(self):
        return self.re.search(self.name_re, self.get_url()).group(1)


main = MangaKatanaCom



import json
import webbrowser

from packaging import version
from requests import get

from manga_py.meta import version, repo_name


def check_version():
    api_url = 'https://api.github.com/repos/' + repo_name + '/releases/latest'
    api_content = json.loads(get(api_url).text)
    tag_name = api_content.get('tag_name', None)
    if tag_name and version.parse(tag_name) > version.parse(version):
        download_addr = api_content['assets'][0]
        return tag_name, download_addr['browser_download_url']
    return ()


def download_update():
    pass


def open_browser(url):
    webbrowser.open(url)



from .adulto_seinagi_org import AdultoSeinagiOrg


class ChouTensei260mbNet(AdultoSeinagiOrg):
    def prepare_cookies(self):
        self._storage['cookies'][' __test'] = '9f148766d926b07a85683d7a6cd50150'
        super().prepare_cookies()


main = ChouTensei260mbNet
